if (!astun) { astun = {}; }
if (!astun.responsiveDesign) { astun.responsiveDesign = {}; }

// add a getElementsByClassName method
(function() {
	var indexOf = [].indexOf || function(prop) {
		for (var i = 0; i < this.length; i++) {
			if (this[i] === prop) return i;
		}
		return -1;
	};
	window.getElementsByClassName = function(className,context) {
		if (context.getElementsByClassName) return context.getElementsByClassName(className);
		var elems = document.querySelectorAll ? context.querySelectorAll("." + className) : (function() {
			var all = context.getElementsByTagName("*"),
				elements = [],
				i = 0;
			for (; i < all.length; i++) {
				if (all[i].className && (" " + all[i].className + " ").indexOf(" " + className + " ") > -1 && indexOf.call(elements,all[i]) === -1) elements.push(all[i]);
			}
			return elements;
		})();
		return elems;
	};
})();

astun.responsiveDesign.getViewportSize = function() {
	return { 'width' : document.documentElement.clientWidth, 'height' : document.documentElement.clientHeight };
};

astun.responsiveDesign.getHeight = function(element) {
	if (!element) return;
	
	return element.style.height !== '' ? element.style.height : element.offsetHeight;
};

astun.responsiveDesign.getWidth = function(element) {
	if (!element) return;
	
	return element.style.width !== '' ? element.style.width : element.offsetWidth;
};

astun.responsiveDesign.getSize = function(element) {
	if (!element) return;
	
	return { 'width' : astun.responsiveDesign.getWidth(element), 'height' : astun.responsiveDesign.getHeight(element) };
};		

astun.responsiveDesign.rearrangePanels = function() {
	// set the minimum panel width
	var minimumPanelWidth = 300;
	// set the maximum number of columns
	var maximumNumberOfColumns = 3;
	
	// exit if we are on the my nearest tab
	if (document.getElementById('atMyNearest'))
		return;
	
	// get the content div
	var contentArea = document.getElementById('atContent');
	if (!contentArea) 
		return;

	// get the content area size
	var contentAreaSize = astun.responsiveDesign.getSize(contentArea);
	
	// get the panel container div
	var panelContainers = window.getElementsByClassName('atPanelContainer', contentArea);
	if (!panelContainers || typeof(panelContainers) === 'string' || !panelContainers.length || panelContainers.length === 0)
		return;
	
	// get a list of the panels
	var panels = window.getElementsByClassName('atPanel', contentArea);
	if (!panels || typeof(panels) === 'string' || !panels.length || panels.length === 1)
		return;
		
	// get a count of the panels
	var numberOfPanels = panels.length;
	
	// loop round the panels and get all of the heights
	var panelHeights = [];
	var totalPanelHeight = 0;
	for (var i = 0; i < numberOfPanels; i++) {
		var height = astun.responsiveDesign.getHeight(panels[i]);
		panelHeights.push(height);
		totalPanelHeight += height;
	}
	
	// decide how many columns we want
	var numberOfColumns = parseInt(contentAreaSize.width / minimumPanelWidth);
	if (numberOfColumns > numberOfPanels) 
		numberOfColumns = numberOfPanels;
	if (numberOfColumns > maximumNumberOfColumns)
		numberOfColumns = maximumNumberOfColumns;
	
	// decide how wide the columns should actually be
	var columnWidth = parseInt((contentAreaSize.width - 10) / numberOfColumns);
	
	// decide how tall the columns should be
	var maximumColumnHeight = parseInt(totalPanelHeight / numberOfColumns);
	
	// create the columns containers
	var columns = [], columnHeights = [];
	for (var i = 0; i < numberOfColumns; i++) {
		columns.push(document.createElement('div'));
		columns[i].setAttribute('class', 'column');
		if (numberOfColumns > 1)
			columns[i].setAttribute('style', 'width:' + columnWidth + 'px');
		columnHeights.push(0);
	}
	
	// build up the columns
	if (panels.length > numberOfColumns) {
		var column = 0;
		while (panels.length > 0) {
			if (columnHeights[column] > 0 && columnHeights[column] + panelHeights[0] > maximumColumnHeight && column < numberOfColumns - 1)
				column++;
		
			columns[column].appendChild(panels[0]);
			columnHeights[column] += panelHeights[0];
			panelHeights.splice(0, 1);
		}
	} else {
		/* special case for 1 panel per column, just to make sure it does as we want */
		for (var i = 0; i < numberOfColumns; i++) {
			columns[i].appendChild(panels[0]);
		}
	}
	
	// remove any existing columns from the DOM
	var removableColumns = window.getElementsByClassName('column', panelContainers[0]);
	if (removableColumns && typeof removableColumns !== 'string' && removableColumns.length && removableColumns.length > 0) {
		while (removableColumns.length > 0) {
			panelContainers[0].removeChild(removableColumns[0]);
		}
	}
	
	// add the columns to the DOM
	for (var i = 0; i < columns.length; i++)
		panelContainers[0].appendChild(columns[i]);
		
	// get rid of any surplus panel containers
	if (panelContainers.length > 1) {
		while (panelContainers.length > 1) {
			panelContainers.parentNode.removeChild(panelContainers[i]);
		}
	}
};

astun.responsiveDesign.doResize = function() {
	astun.responsiveDesign.rearrangePanels();

	if (typeof originalOnResize === 'function') {
		// call the original event
		window._onresize();
	}
};

// store the original function so it's safe
window._onresize = window.onresize;
window._resizeTimeoutId = null;

window.onresize = function(event) {
	if (window._resizeTimeoutId !== null)
		window.clearTimeout(window._resizeTimeoutId);

	window._resizeTimeoutId = window.setTimeout(astun.responsiveDesign.doResize, 10);
};



astun.responsiveDesign.rearrangePanels();	